package com.seeyon.apps.ocip.exchange.edoc;

import java.io.*;
import java.text.SimpleDateFormat;
import java.util.*;

import com.seeyon.apps.ocip.common.dao.OcipPropertiesDao;
import com.seeyon.apps.ocip.common.po.OcipProperties;
import com.seeyon.ctp.util.IOUtility;
import com.seeyon.ctp.util.json.JSONUtil;
import com.seeyon.ocip.exchange.model.*;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.io.FileUtils;
import org.apache.commons.io.IOUtils;
import org.apache.commons.logging.Log;

import com.seeyon.apps.exchange.engine.service.ExchangeService;
import com.seeyon.apps.govdoc.constant.GovdocEnum;
import com.seeyon.apps.govdoc.helper.GovdocContentHelper;
import com.seeyon.apps.govdoc.manager.GovdocExchangeManager;
import com.seeyon.apps.govdoc.manager.impl.RecieveGovdocObjManager;
import com.seeyon.apps.govdoc.po.GovdocExchangeDetail;
import com.seeyon.apps.govdoc.po.GovdocExchangeMain;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.ocip.util.CommonUtil;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.mainbody.MainbodyService;
import com.seeyon.ctp.common.content.mainbody.MainbodyType;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.filemanager.Constants;
import com.seeyon.ctp.common.filemanager.manager.AttachmentManager;
import com.seeyon.ctp.common.filemanager.manager.FileManager;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.content.CtpContentAll;
import com.seeyon.ctp.common.po.filemanager.Attachment;
import com.seeyon.ctp.common.po.filemanager.V3XFile;
import com.seeyon.ctp.organization.bo.V3xOrgEntity;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.bo.V3xOrgUnit;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ocip.common.IConstant.AddressType;
import com.seeyon.ocip.common.entry.Address;
import com.seeyon.ocip.common.org.OcipOrgUnit;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.exchange.api.IBussinessHandler;
import com.seeyon.ocip.exchange.exceptions.BussinessException;
import com.seeyon.ocip.exchange.model.edoc.OFCEdocObject;
import com.seeyon.ocip.exchange.model.edoc.SeeyonEdoc;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;

public class EdocOFCExchangeHandler implements IBussinessHandler{
	
	private static final Log LOGGER = CtpLogFactory.getLog(EdocOFCExchangeHandler.class);
	private OrgManager orgManager;
	private AttachmentManager attachmentManager;
	private FileManager fileManager;
	private RecieveGovdocObjManager recieveGovdocObjManager;
	private GovdocExchangeManager govdocExchangeManager;
	private IOrganizationManager organizationManager;
	private OcipPropertiesDao ocipPropertiesDao;

	public void setOcipPropertiesDao(OcipPropertiesDao ocipPropertiesDao) {
		this.ocipPropertiesDao = ocipPropertiesDao;
	}

	public void setOrganizationManager(IOrganizationManager organizationManager) {
		this.organizationManager = organizationManager;
	}
	public void setGovdocExchangeManager(GovdocExchangeManager govdocExchangeManager) {
		this.govdocExchangeManager = govdocExchangeManager;
	}
	public RecieveGovdocObjManager getRecieveGovdocObjManager() {
		return recieveGovdocObjManager;
	}
	public void setRecieveGovdocObjManager(
			RecieveGovdocObjManager recieveGovdocObjManager) {
		this.recieveGovdocObjManager = recieveGovdocObjManager;
	}

	public void setFileManager(FileManager fileManager) {
		this.fileManager = fileManager;
	}

	public void setAttachmentManager(AttachmentManager attachmentManager) {
		this.attachmentManager = attachmentManager;
	}

	public void setOrgManager(OrgManager orgManager) {
		this.orgManager = orgManager;
	}

	@Override
	public List<BussinessResult> exchangeReceive(BIZExchangeData bizData)
			throws BussinessException {
		List<BussinessResult> bussinessResults = new ArrayList<BussinessResult>();
		List<Organization> reciList = bizData.getRecivers();
		List<Organization> localRecList = new ArrayList<Organization>();
		for (Organization organization2 : reciList) {
			BussinessResult result = new BussinessResult();
			result.setCode("0");
			result.setIdentifier(bizData.getIdentifier());
			result.setOrganization(organization2);
			result.setMessage("公文接收成功");
			bussinessResults.add(result);
			//过滤本系统接收单位
			String localObjectId = organizationManager.getLocalObjectId(organization2.getIdentification());
			if(localObjectId!=null){
				localRecList.add(organization2);
			}
			bizData.setRecivers(localRecList);
		}
		try {
			BIZMessage bizMessage = bizData.getBussnissMessage();
			Address source = bizData.getSource();
			OFCEdocObject ofcEdocObject = (OFCEdocObject)bizData.getBussnissMessage().getContent();
			SeeyonEdoc seeyonEdoc = (SeeyonEdoc)ofcEdocObject.getExtendAttr();
			Long summaryId = Long.parseLong(seeyonEdoc.getDocumentIdentifier());

			Address creater = bizData.getSender().getIdentification();
			Long createId = Long.valueOf(organizationManager.getPlatformId(creater));
			Long createunitId = Long.parseLong(ofcEdocObject.getIssueOrganization());

			//创建govdocExchangeMain
			String sendCode = source.getId();
			Address address = new Address();
			address.setId(sendCode);
			address.setName(sendCode);
			address.setResource(sendCode);
			address.setType(AddressType.system.name());

			//存储来问信息
			//获取文号
			PropertyValue edocMarkPropertyValue = seeyonEdoc.getDocMark();
			String edocMark = edocMarkPropertyValue.getDisplay();
			String edocTitle = seeyonEdoc.getSubject();
			Long  sendSummaryId = Long.parseLong(seeyonEdoc.getDocumentIdentifier());
			List<OcipProperties> ocipPropertiesList = ocipPropertiesDao.findListBySourceIds("edoc", sendSummaryId);
			if(ocipPropertiesList == null  || ocipPropertiesList.isEmpty())
			{
				OcipProperties ocipEdocProperties = new OcipProperties();
				ocipEdocProperties.setSourceId(sendSummaryId);
				ocipEdocProperties.setType("edoc");
				ocipEdocProperties.setName(edocTitle);
				ocipEdocProperties.setOrgAccountId(createunitId);
				Map<String,Object>  edocValue = new HashMap<String, Object>();
				edocValue.put("edocMark",edocMark);
				edocValue.put("edocTitle",edocTitle);


				SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd hh:mm");
				edocValue.put("sendTime",sdf.format(new Date()));
				ocipEdocProperties.setValue(JSONUtil.toJSONString(edocValue));
				ocipEdocProperties.setIdIfNew();
				ocipPropertiesDao.save(ocipEdocProperties);
			}

			GovdocExchangeMain govdocExchangeMain = createGovdocExchangeMain(bizData,createId,seeyonEdoc,address.toString());
			String sendUnitName = seeyonEdoc.getSendUnit();
			LOGGER.info("创建附件createAttachment");
			//创建附件
			createAttachment(bizMessage,summaryId);
			LOGGER.info("创建正文createCtpContentAll");
			//创建正文
			createCtpContentAll(bizMessage,summaryId,createId);
			//创建detail
			createGovdocExchangeDetail(bizData, govdocExchangeMain,createunitId,sendUnitName);
			
			User user = new User();
			V3xOrgMember member = orgManager.getMemberById(createId);
			LOGGER.info("创建人员" + member.getId());
			user.setId(createId);
			user.setLoginAccount(member.getOrgAccountId());
			user.setAccountId(member.getOrgAccountId());
			user.setDepartmentId(member.getOrgDepartmentId());
			user.setName(member.getName());
			CommonUtil.setCurrentUser(user);
			bizData.getSender().getIdentification().setId(createId.toString());
			bizData.getSender().getIdentification().setResource("0");

			ExchangeService.sendObject(bizData);
			
			//recieveGovdocObjManager.send(bizData);
		} catch (Exception e) {
			String msg = "ocip接收公文时出错";
			LOGGER.error(msg,e);
			for (BussinessResult bussinessResult : bussinessResults) {
				bussinessResult.setCode("2");
				bussinessResult.setMessage(msg);
			}
		}
		return bussinessResults;
	}

	@Override
	public BIZExchangeData exchangeSend(Map<String, Object> params)
			throws BussinessException {
		BIZExchangeData bizData = (BIZExchangeData)params.get("bizData");
		OFCEdocObject ofcObject  = (OFCEdocObject)bizData.getBussnissMessage().getContent();
		SeeyonEdoc seeyonEdoc = (SeeyonEdoc)ofcObject.getExtendAttr();
		List<Organization> organizations = bizData.getRecivers();
		List<Organization> recivers = new ArrayList<Organization>();
		User user = (User)params.get("user");
		OcipOrgUnit unit = organizationManager.getAccount(user.getAccountId().toString(), bizData.getSource().getId());
		if(unit!=null)
			ofcObject.setIssueOrganization(unit.getId().toString());
		//过滤外单位接受者
		for (Organization organization : organizations) {
			Address address = organization.getIdentification();
			
			try {
				V3xOrgEntity entity = orgManager.getEntityOnlyById(Long.parseLong(address.getId()));
				if(entity!=null&&OrgUtil.isPlatformEntity(entity)){
					recivers.add(organization);
				}
			} catch (Exception e) {
				LOGGER.error(e);
			}
		}
		//发送者设置为平台对象
		Organization organization = bizData.getSender();
		Address addres = organization.getIdentification();
		addres.setId(organizationManager.getPlatformId(addres));
		addres.setResource("0");
		//
		bizData.setRecivers(recivers);
		Long summaryId = Long.parseLong(seeyonEdoc.getDocumentIdentifier());
		bizData.setGroupId(summaryId.toString());
		//初始化正文
		initGovdocContentAll(summaryId,ofcObject);
		//初始化附件
		initGovdocAttachment(summaryId, bizData.getBussnissMessage());
		
		return bizData;
	}
	//初始化正文
	private void initGovdocContentAll(Long summaryId, OFCEdocObject object){
		CtpContentAll govdocCtpContentAll = GovdocContentHelper.getTransBodyContentByModuleId(summaryId);
		object.setContentType(MainbodyType.getEnumByKey(govdocCtpContentAll.getContentType()).name());
		if(govdocCtpContentAll.getContentType() == MainbodyType.HTML.getKey()){
			try {
				if(govdocCtpContentAll.getContent()!=null){
					object.setContent(govdocCtpContentAll.getContent().getBytes("utf-8"));
				}else{
					object.setContent("".getBytes("utf-8"));
				}
			} catch (UnsupportedEncodingException e) {
				LOGGER.error(e);
			}
		}else{
			Long fileId = Long.parseLong(govdocCtpContentAll.getContent());
			try {
				V3XFile v3xFile = fileManager.getV3XFile(fileId);
				if(v3xFile != null)
				{
					object.setContentMimeType(v3xFile.getMimeType());
					//大文件start
					File file = fileManager.getFile(fileId);
					object.setContentDataHandler(new DataHandler(new FileDataSource(file)));
				}
				else
				{

					object.setContentMimeType(getMimeType(govdocCtpContentAll.getContentType()));
				}

//				//大文件start
//				File file = fileManager.getFile(fileId);
//				object.setContentDataHandler(new DataHandler(new FileDataSource(file)));
				//大文件end
				//老版start
				//object.setContent(fileManager.getFileBytes(fileId));
				//老版end
			} catch (BusinessException e) {
				LOGGER.error("初始化公文交换对象获取公文正文数据出错",e);
			}
		}
	}
	private String getMimeType(int conentType) {
//		HTML(10, "common.body.type.html.label", "<span class=\"ico16 html_16\"></span>"),
//		FORM(20, "表单格式正文", "<span class=\"ico16 form_text_16\"></span>"),
//		CAP4FORM(21, "CAP4表单格式正文", "<span class=\"ico16 form_text_16\"></span>"),
//		TXT(30, "text正文", "<span class=\"ico16 txt_16\"></span>"),
//		OfficeWord(41, "common.body.type.officeword.label", "<span class=\"ico16 doc_16\"></span>"),
//		OfficeExcel(42, "common.body.type.officeexcel.label", "<span class=\"ico16 xls_16\"></span>"),
//		WpsWord(43, "common.body.type.wpsword.label", "<span class=\"ico16 wps_16\"></span>"),
//		WpsExcel(44, "common.body.type.wpsexcel.label", "<span class=\"ico16 xls2_16\"></span>"),
//		Pdf(45, "common_body_type_pdf_label", "<span class=\"ico16 pdf_16\"></span>"),
//		Ofd(46, "common_body_type_ofd_label", "<span class=\"ico16 ofd_16\"></span>");
		if (conentType == 41 || conentType == 43) {
			return "application/msword";
		} else if (conentType == 42 || conentType == 44) {
			return "application/vnd.ms-excel";
		} else if (conentType == 30) {
			return "text/plain";
		} else if (10 == conentType) {
			return "text/html";
		} else if (conentType == 45) {
			return "application/pdf";
		}
		else if (conentType == 46) {
			return "application/ofd";
		}
		else
		{
			return "application/octet-stream";
		}
	}
	//初始化公文附件
	private void initGovdocAttachment(Long summaryId,BIZMessage bizMessage) {
		// 附件加载
		List<AttachmentFile> attfiles = new ArrayList<AttachmentFile>();
		try {
			List<Attachment> attachments = attachmentManager.getByReference(summaryId);
			for (Attachment attachment : attachments) {

				if(summaryId.equals(attachment.getReference())&& summaryId.equals(attachment.getSubReference()) || attachment.getCategory()== ApplicationCategoryEnum.form.getKey()){

					V3XFile v3xFile = fileManager.getV3XFile(attachment.getFileUrl());
					if (v3xFile != null) {
						AttachmentFile att = new AttachmentFile();
						att.setIdentification(v3xFile.getId().toString());
						att.setName(v3xFile.getFilename());
						att.setSize(v3xFile.getSize());
						//大文件start
						att.setDataHandler(new DataHandler(new FileDataSource(fileManager.getFile(v3xFile.getId()))));
						//大文件end
						//att.setData(fileManager.getFileBytes(v3xFile.getId()));
						//老版start
						attfiles.add(att);
						//老版end
					}

				}

			}
			bizMessage.setAttachments(attfiles);
		} catch (Exception e) {
			LOGGER.error("ocip公文交换初始化加载附件出错",e);
		}
	}
	private GovdocExchangeMain createGovdocExchangeMain(BIZExchangeData bizData ,Long createId ,SeeyonEdoc seeyonEdoc,String ocipSysCode) throws BusinessException {
		Long mainId = Long.parseLong(seeyonEdoc.getMainId());
		Long summaryId = Long.parseLong(seeyonEdoc.getDocumentIdentifier());
		GovdocExchangeMain govdocExchangeMain = govdocExchangeManager.getGovdocExchangeMainById(mainId);
		if (govdocExchangeMain == null) {
			try {
				//5.6的数据发送record，补发会重新生成main。这里判断下
				GovdocExchangeMain main = govdocExchangeManager.findBySummaryId(summaryId, null);
				if(main!=null){
					return main;
				}
				govdocExchangeMain = new GovdocExchangeMain();
				govdocExchangeMain.setId(mainId);
				govdocExchangeMain.setSubject(bizData.getSubject());
				govdocExchangeMain.setCreateTime(new Date());
				govdocExchangeMain.setStartUserId(createId);
				govdocExchangeMain.setStartTime(new Date());
				govdocExchangeMain.setSummaryId(summaryId);
				govdocExchangeMain.setOcipSysCode(ocipSysCode);
				govdocExchangeMain.setType(GovdocExchangeMain.EXCHANGE_TYPE_JIAOHUAN);
				govdocExchangeMain.setCreatePerson(String.valueOf(createId));// 增加交换创建者id,区分接收还是发送
				govdocExchangeManager.saveOrUpdateMain(govdocExchangeMain);
			} catch (BusinessException e) {
				LOGGER.error("公文交换接收方接收数据处理失败！-001\n relationId=" + summaryId + " affairId=", e);
				throw new BusinessException("公文交换接收方接收数据处理失败！-001\n");
			}
		}
		return govdocExchangeMain;
	}
	private List<GovdocExchangeDetail> createGovdocExchangeDetail(BIZExchangeData bd, GovdocExchangeMain govdocExchangeMain,Long createunitId,String sendUnitName) throws BusinessException {
		List<GovdocExchangeDetail> govdocDetails = new ArrayList<GovdocExchangeDetail>();
		OFCEdocObject obj = (OFCEdocObject)bd.getBussnissMessage().getContent();
		bd.getSender();
		SeeyonEdoc seeyonEdoc = (SeeyonEdoc)obj.getExtendAttr();
		Map<Long,Long> details = seeyonEdoc.getDetailIds();
		Object receiveUnitObj = bd.getRecivers();
		List<Organization> receiveUnit = (List<Organization>)receiveUnitObj;
		//循环 接受单位，匹配detail的对应单位并保存detail
		for (Organization org : receiveUnit) {
			Address address = org.getIdentification();
			V3xOrgUnit orgUnit = null;
			try {
				orgUnit = getLocalOrgUnit(address);
				if (orgUnit != null) {
					Long detailId = Long.valueOf(String.valueOf(details.get(Long.parseLong(address.getId()))));
					GovdocExchangeDetail govdocExchangeDetail = govdocExchangeManager.getExchangeDetailById(detailId);
					if (govdocExchangeDetail == null) {
						govdocExchangeDetail = new GovdocExchangeDetail();
						govdocExchangeDetail.setId(detailId);
						govdocExchangeDetail.setMainId(govdocExchangeMain.getId());
						govdocExchangeDetail.setRecOrgId(orgUnit.getId());
						String type = orgUnit.getType().toString();
						if (GovdocEnum.GovdocExchangeOrgType.department.name().equalsIgnoreCase(type)) {
							govdocExchangeDetail.setRecOrgType(GovdocEnum.GovdocExchangeOrgType.department.getKey());
							govdocExchangeDetail.setRecAccountId(orgUnit.getOrgAccountId());
						} else if (GovdocEnum.GovdocExchangeOrgType.account.name().equalsIgnoreCase(type)) {
							govdocExchangeDetail.setRecOrgType(GovdocEnum.GovdocExchangeOrgType.account.getKey());
							govdocExchangeDetail.setRecAccountId(orgUnit.getId());
						}
						govdocExchangeDetail.setRecOrgName(orgUnit.getName());
						govdocExchangeDetail.setStatus(GovdocEnum.ExchangeDetailStatus.waitSign.getKey());
						govdocExchangeDetail.setCreateTime(new Date());
						govdocExchangeDetail.setSendAccountId(createunitId);
						govdocExchangeDetail.setSendUnit(sendUnitName);
//						govdocExchangeManager.updateDetail(govdocExchangeDetail);
					}
					//每次创建detail时，传入对应报文的id
					Map<String,Object> map = GovdocUtil.getExtProperty(govdocExchangeDetail);
					map.put("identifier", bd.getIdentifier());
					GovdocUtil.setExtProperty(govdocExchangeDetail, map);
					govdocExchangeManager.updateDetail(govdocExchangeDetail);
					govdocDetails.add(govdocExchangeDetail);
				} else {
					LOGGER.info("本地未找到匹配机构，将跳过，address=" + address);
				}
			} catch (Exception e) {
				LOGGER.error("公文交换接收方接收数据生成签收数据处理失败！-001\n", e);
				throw new BusinessException("公文交换接收方接收数据生成签收数据处理失败！-001\n");
			}

		}
		return govdocDetails;
	}
	private void createCtpContentAll(BIZMessage bizMessage,
			Long relationId,Long createId) {
		// 特殊处理OFD文件，默认生成代签收，正文为world类型
		OFCEdocObject ofcEdocObject = (OFCEdocObject)bizMessage.getContent();
		MainbodyType[] mainbodyTypes = MainbodyType.values();
		MainbodyType mainbodyType = null;
		for (MainbodyType type : mainbodyTypes) {
			if (type.name().equalsIgnoreCase(ofcEdocObject.getContentType())) {
				mainbodyType = type;
				break;
			}
		}
		if (mainbodyType == null) {
			LOGGER.error(String.format("不能解析的文件类型（%s），将正文作为附件保存", ofcEdocObject.getContentType()));
		}
		try {

				Date now = new Date();
				CtpContentAll ctpContentAll = new CtpContentAll();
				ctpContentAll.setIdIfNew();
				ctpContentAll.setModuleId(relationId);
				ctpContentAll.setCreateDate(new Date());
				ctpContentAll.setModuleType(ApplicationCategoryEnum.edoc.getKey());
				ctpContentAll.setCreateId(createId);
				ctpContentAll.setModifyId(createId);
				ctpContentAll.setSort(1);
				ctpContentAll.setContentTemplateId(0l);
				ctpContentAll.setModuleTemplateId(0l);
				if(MainbodyType.HTML.equals(mainbodyType)){
					ctpContentAll.setContentType(MainbodyType.HTML.getKey());
					ctpContentAll.setContent(new String(ofcEdocObject.getContent(),"utf-8"));
					MainbodyService.getInstance().getContentManager().saveOrUpdateContentAll(ctpContentAll);
				}else{
//					BufferedOutputStream stream = null;
					//大文件start
					OutputStream output = null;
					//大文件end
					try{
						Long fileid = UUIDLong.longUUID();
						String srcPath = fileManager.getFolder(now, true) + java.io.File.separator + fileid;
						File file = new File(srcPath);
						//老版start
//						FileUtils.writeByteArrayToFile(file, ofcEdocObject.getContent());
						//老版end

						//大文件start
						DataHandler dataHandler = ofcEdocObject.getContentDataHandler();

						if (dataHandler != null) {
							output = FileUtils.openOutputStream(file);
							dataHandler.writeTo(output);
							IOUtils.closeQuietly(output);
						} else {
							if(ofcEdocObject.getContent() != null)
							{
								output = FileUtils.openOutputStream(file);
								IOUtility.copy(ofcEdocObject.getContent(), output);
								IOUtils.closeQuietly(output);
							}

						}

						//大文件end

						V3XFile v3xFile = new V3XFile(fileid);
						v3xFile.setCategory(ApplicationCategoryEnum.edoc.key());
						v3xFile.setType(Constants.ATTACHMENT_TYPE.FILE.ordinal());
						v3xFile.setFilename(fileid.toString());
						if (MainbodyType.Ofd.equals(mainbodyType)) {
							v3xFile.setMimeType("application/ofd");
							v3xFile.setFilename(ofcEdocObject.getTitle() + "." + String.valueOf(ofcEdocObject.getContentType()).toLowerCase());
						} else {
							v3xFile.setMimeType(ofcEdocObject.getContentMimeType());
						}
						v3xFile.setCreateDate(now);
						v3xFile.setSize(file.length());// 文件大小
						v3xFile.setUpdateDate(now);
						fileManager.save(v3xFile);
						if (mainbodyType != null) {
							ctpContentAll.setContentType(mainbodyType.getKey());
							ctpContentAll.setContent(v3xFile.getId().toString());
//							if (mainbodyType == MainbodyType.Ofd) {
//								ctpContentAll.setContentType(MainbodyType.HTML.getKey());
//								ctpContentAll.setContent("");
//								ofcEdocObject.setContent("".getBytes("UTF-8"));
//								ofcEdocObject.setContentType(MainbodyType.HTML.name());
//								attachmentManager.create(new Long[] { v3xFile.getId() }, ApplicationCategoryEnum.edoc, relationId, relationId);
//								AttachmentFile att = new AttachmentFile();
//								att.setIdentification(v3xFile.getId().toString());
//								bizMessage.getAttachments().add(att);
//							}
							MainbodyService.getInstance().getContentManager().saveOrUpdateContentAll(ctpContentAll);
						} else {
							attachmentManager.create(new Long[] { v3xFile.getId() }, ApplicationCategoryEnum.edoc, relationId, relationId);
							AttachmentFile att = new AttachmentFile();
							att.setIdentification(v3xFile.getId().toString());
							bizMessage.getAttachments().add(att);
						}

					}catch(Exception e){
						LOGGER.error("ocip公文交换生成正文时，创建文件信息失败",e);
					}
				}

		} catch (Exception e) {
			LOGGER.error("ocip公文交换生成正文数据时异常",e);
		}
	}
	private V3xOrgUnit getLocalOrgUnit(Address address) {
		V3xOrgUnit orgUnit = null;
		try {
			String localObjectId = organizationManager.getLocalObjectId(address);
			if (localObjectId != null) {
				orgUnit = orgManager.getUnitById(Long.parseLong(localObjectId));
			}
		} catch (Exception e) {
			LOGGER.error("平台单位信息转换为本系统单位信息失败", e);
		}
		return orgUnit;
	}
	private void createAttachment(BIZMessage bussnissMessage, Long relationId){
		List<AttachmentFile> atts = bussnissMessage.getAttachments();
		if(CollectionUtils.isNotEmpty(atts)){
			Date now = new Date();
			for (AttachmentFile att : atts) {
				BufferedOutputStream stream = null;
				FileOutputStream outputStream = null;
				try{
					String srcPath = fileManager.getFolder(now, true) + java.io.File.separator + att.getIdentification();
					File file = new File(srcPath);
					outputStream = new FileOutputStream(file);
					stream  = new BufferedOutputStream(outputStream);
					//大文件start
					DataHandler dataHandler = att.getDataHandler();
					if (dataHandler != null) {
						dataHandler.writeTo(stream);
					}else{
						stream.write(att.getData());
					}
					//大文件end
//					stream.write(att.getData());
					
					V3XFile v3xFile = new V3XFile(Long.parseLong(att.getIdentification()));
					v3xFile.setCategory(ApplicationCategoryEnum.edoc.key());
					v3xFile.setType(Constants.ATTACHMENT_TYPE.FILE.ordinal());
					v3xFile.setFilename(att.getName());
					v3xFile.setMimeType("text/plain");// 附件类型
					v3xFile.setCreateDate(now);
					v3xFile.setSize(file.length());// 文件大小
					v3xFile.setUpdateDate(now);
					// 保存正文
					fileManager.save(v3xFile);
					Long[] fileids = new Long[]{v3xFile.getId()};
					attachmentManager.create(fileids, ApplicationCategoryEnum.edoc, relationId, relationId);
				}catch(Exception e){
					LOGGER.error("ocip公文交换接收方生成附件文件出错",e);
				}finally{
					try{
						if(stream!=null){
							stream.close();	
						}
						if(outputStream != null){						
							outputStream.close();											
						}
					} catch(IOException e){
						LOGGER.error(e);
					}					
				}	
			}
		}
	}
	@Override
	public BIZContentType type() {
		return BIZContentType.OFC;
	}
	
}
