package com.seeyon.apps.ocip.exchange.edoc;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import com.seeyon.apps.govdoc.constant.GovdocEnum;
import com.seeyon.apps.govdoc.constant.GovdocEnum.ExchangeDetailStatus;
import com.seeyon.apps.govdoc.manager.GovdocExchangeManager;
import com.seeyon.apps.govdoc.manager.GovdocManager;
import com.seeyon.apps.govdoc.manager.GovdocSummaryManager;
import com.seeyon.apps.govdoc.po.GovdocExchangeDetail;
import com.seeyon.apps.govdoc.po.GovdocExchangeDetailLog;
import com.seeyon.apps.govdoc.po.GovdocExchangeMain;
import com.seeyon.apps.govdoc.util.GovdocUtil;
import com.seeyon.apps.govdoc.vo.GovdocRepealVO;
import com.seeyon.apps.ocip.util.CommonUtil;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.usermessage.MessageContent;
import com.seeyon.ctp.common.usermessage.MessageReceiver;
import com.seeyon.ctp.common.usermessage.UserMessageManager;
import com.seeyon.ctp.common.usermessage.UserMessageUtil;
import com.seeyon.ctp.organization.bo.V3xOrgEntity;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UUIDLong;
import com.seeyon.ctp.workflow.wapi.WorkflowApiManager;
import com.seeyon.ocip.common.IConstant;
import com.seeyon.ocip.common.IConstant.AddressType;
import com.seeyon.ocip.common.entry.Address;
import com.seeyon.ocip.common.org.OcipOrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.common.utils.Global;
import com.seeyon.ocip.exchange.api.IBussinessHandler;
import com.seeyon.ocip.exchange.exceptions.BussinessException;
import com.seeyon.ocip.exchange.model.BIZContentType;
import com.seeyon.ocip.exchange.model.BIZExchangeData;
import com.seeyon.ocip.exchange.model.BIZMessage;
import com.seeyon.ocip.exchange.model.BussinessResult;
import com.seeyon.ocip.exchange.model.Organization;
import com.seeyon.ocip.exchange.model.edoc.EdocOperation;
import com.seeyon.ocip.exchange.model.edoc.RETEdocObject;
import com.seeyon.v3x.edoc.domain.EdocSummary;

public class EdocRETExchangeHandler implements IBussinessHandler{
	private static final Log LOGGER = LogFactory.getLog(EdocRETExchangeHandler.class);
	private GovdocExchangeManager govdocExchangeManager;
	private OrgManager orgManager;
	private IOrganizationManager organizationManager;
	private AffairManager affairManager;
	private UserMessageManager userMessageManager;
	private GovdocManager govdocManager;
	private GovdocSummaryManager govdocSummaryManager;
	private WorkflowApiManager wapi;

	public void setGovdocManager(GovdocManager govdocManager) {
		this.govdocManager = govdocManager;
	}

	public void setGovdocSummaryManager(GovdocSummaryManager govdocSummaryManager) {
		this.govdocSummaryManager = govdocSummaryManager;
	}

	public void setUserMessageManager(UserMessageManager userMessageManager) {
		this.userMessageManager = userMessageManager;
	}

	public void setAffairManager(AffairManager affairManager) {
		this.affairManager = affairManager;
	}

	public void setOrganizationManager(IOrganizationManager organizationManager) {
		this.organizationManager = organizationManager;
	}

	public void setOrgManager(OrgManager orgManager) {
		this.orgManager = orgManager;
	}

	public void setGovdocExchangeManager(GovdocExchangeManager govdocExchangeManager) {
		this.govdocExchangeManager = govdocExchangeManager;
	}

	@Override
	public BIZContentType type() {
		return BIZContentType.RET;
	}

	@Override
	//发送公文方执行操作
	public List<BussinessResult> exchangeReceive(BIZExchangeData in)
			throws BussinessException {
		List<BussinessResult> bussinessResults = new ArrayList<BussinessResult>();
		BIZMessage bussnissMessage = in.getBussnissMessage();
		Organization organization = in.getSender();
		Address creater = organization.getIdentification();
		BIZContentType contentType = bussnissMessage.getContentType();
		List<Organization> reciList = in.getRecivers();
		for (Organization organization2 : reciList) {
			BussinessResult result = new BussinessResult();
			result.setCode("0");
			result.setIdentifier(in.getIdentifier());
			result.setOrganization(organization2);
			result.setMessage("公文回执消息接受成功");
			bussinessResults.add(result);
		}
		if(BIZContentType.RET.equals(contentType)){
			RETEdocObject retEdocObject = (RETEdocObject)bussnissMessage.getContent();
			try {
				GovdocExchangeDetail detail = govdocExchangeManager.getExchangeDetailById(Long.parseLong(retEdocObject.getDetailId()));
				if(detail == null)
				{
					return bussinessResults;
				}
				GovdocExchangeDetailLog detailLog = new GovdocExchangeDetailLog();
				if(retEdocObject.getOperation().equals(EdocOperation.RECEIVED)){//收到公文交换后回执
					GovdocExchangeMain main = govdocExchangeManager.getGovdocExchangeMainById(detail.getMainId());
					String ocipExchangeStatus = retEdocObject.getExchangeStatus();
					
					detailLog.setDetailId(detail.getId());
					Calendar calendar = Calendar.getInstance();
					Date date = new Date();
					calendar.setTime(date);
					detailLog.setTime(date);
					detailLog.setTimeMS(calendar.get(Calendar.MILLISECOND));
					detailLog.setIdIfNew();
					V3xOrgMember member = orgManager.getMemberById(main.getStartUserId());
					if(member!=null)
						detailLog.setUserName(member.getName());
					detail.setStatus(GovdocEnum.ExchangeDetailStatus.waitSend.getKey());
					detailLog.setStatus(ExchangeDetailStatus.waitSend.getKey());
					if("0".equals(ocipExchangeStatus)){
						detail.setStatus(GovdocEnum.ExchangeDetailStatus.waitSign.getKey());
						detailLog.setStatus(ExchangeDetailStatus.waitSign.getKey());
						detailLog.setDescription("已分送，等待签收");
					}else if("1".equals(ocipExchangeStatus)){
						detailLog.setDescription("发送失败，收文单位未设置收文交换流程");
					}else if("2".equals(ocipExchangeStatus)){
						detailLog.setDescription("发送失败，收文单位的交换流程被停用或者删除");
					}else if("3".equals(ocipExchangeStatus)){
						detailLog.setDescription("发送失败，接收单位没有找到对应人员");
					}else if("4".equals(ocipExchangeStatus)){
						detailLog.setDescription("交换模板触发失败");
					}
//					if(!ocipExchangeStatus.equals("0")){
//						for (BussinessResult result : bussinessResults) {
//							result.setCode("3");
//							result.setMessage(detailLog.getDescription());
//						}
//					}
					Map<String,Object> map = GovdocUtil.getExtProperty(detail);
					map.put("ocipSyscode", retEdocObject.getOcipSyscode());
					GovdocUtil.setExtProperty(detail, map);
				}else{
					detailLog.setDetailId(detail.getId());
					detailLog.setId(UUIDLong.longUUID());
					Calendar calendar = Calendar.getInstance();
					Date date = new Date();
					calendar.setTime(date);
					detailLog.setTime(date);
					detailLog.setTimeMS(calendar.get(Calendar.MILLISECOND));
					OcipOrgMember ocipMember = organizationManager.getMember(creater.getId(), creater.getResource());
					detail.setOpinion(retEdocObject.getOpinion());
					if(retEdocObject.getOperation().equals(EdocOperation.ACCEPTED)){//签收回执
						V3xOrgMember createMember= orgManager.getMemberById(Long.parseLong(ocipMember.getId()));
						detail.setStatus(GovdocEnum.ExchangeDetailStatus.hasSign.getKey());
						detail.setRecTime(new Date());
						detail.setRecNo(retEdocObject.getRecNo());
						detail.setRecUserName(ocipMember.getName());
						detail.setRecUserId(Long.parseLong(ocipMember.getId()));
						detailLog.setStatus(GovdocEnum.ExchangeDetailStatus.hasSign.getKey());
						detailLog.setUserName(ocipMember.getName());
						detailLog.setDescription("已签收，等待分办");
						String backInfo = retEdocObject.getOpinion();
						GovdocExchangeMain main = govdocExchangeManager.getGovdocExchangeMainById(detail.getMainId());
						CtpAffair affair = affairManager.get(main.getAffairId());
						
						UserMessageManager userMessageManager = (UserMessageManager) AppContext.getBean("userMessageManager");
						MessageReceiver messageReceiver = new MessageReceiver(affair.getId(),main.getStartUserId(),"message.link.ocip.detail",detail.getId());
						// 0-意见隐藏，-1内容为空，1有内容
						int opinionType = 1;
						if (Strings.isBlank(backInfo) || "null".equalsIgnoreCase(backInfo)) {
							opinionType = -1;
						}
						// -1:无附件或意见被隐藏; 1: 无内容（内容前面加“意见：”）; 2:有内容，有附件
						int opinionAtt = (Strings.isBlank(backInfo) ? 1 : 2);
						// 有内容，有附件：减少4个字节
						int deviation = opinionAtt == 2 ? -4 : 0;
						String opinionContent = UserMessageUtil.getComment4Message(backInfo, deviation);
						
						//{0}处理了事务《{1}{2,choice,0#|1# (由{3}原发)}》。
						//{6,choice,-1#|1#[已阅]|2#[同意]|3#[不同意]} {8,choice,0#|1#赞！} {4,choice,-1#|0# 意见被隐藏|1#意见：{5}|2# {5}}{7,choice,-1#|1#意见：(附)|2#(附)}
						MessageContent mc = new MessageContent("govdoc.opinion.deal", createMember.getName(), affair.getSubject(), 0, null, opinionType, opinionContent, -1, -1, 0);
						mc.setImportantLevel(affair.getImportantLevel());
						mc.setBody(backInfo, com.seeyon.ctp.common.constants.Constants.EDITOR_TYPE_HTML, new Date());
						userMessageManager.sendSystemMessage(mc, ApplicationCategoryEnum.edoc, createMember.getId(), messageReceiver);
					}
					if(retEdocObject.getOperation().equals(EdocOperation.REVOKED)){//撤销回执
						V3xOrgMember v3xmember = orgManager.getMemberById(Long.parseLong(ocipMember.getId()));
						User user = new User();
						user.setId(Long.parseLong(ocipMember.getId()));
						user.setLoginAccount(v3xmember.getOrgAccountId());
						user.setAccountId(v3xmember.getOrgAccountId());
						user.setDepartmentId(v3xmember.getOrgDepartmentId());
						user.setName(v3xmember.getName());
						CommonUtil.setCurrentUser(user);
						String affairId = getAffairId(detail);
						// 发起撤销流程
						GovdocRepealVO repealVO = new GovdocRepealVO();
						repealVO.setIsWFTrace("0");
						repealVO.setRepealComment(retEdocObject.getOpinion());
						repealVO.setSummaryIdStr(detail.getSummaryId().toString());
						repealVO.setAffairIdStr(affairId);
						govdocManager.transRepal(repealVO);
					}
					if(retEdocObject.getOperation().equals(EdocOperation.SIGNEDANDSTEPBACK)){
						detail.setStatus(ExchangeDetailStatus.waitSign.getKey());
						detail.setRecNo(null);
						detail.setRecUserName(null);
						detail.setRecUserId(null);
						detail.setRecTime(null);
						detail.setNodeInfo(null);
						detail.setSignAffairId(null);
	    				//记录日志
						detailLog.setDescription("进行了回退操作，交换状态为待签收");
						detailLog.setUserName(ocipMember.getName());
						detailLog.setBackOpinion(retEdocObject.getOpinion());
						detailLog.setStatus(ExchangeDetailStatus.waitSign.getKey());
					}
					if(retEdocObject.getOperation().equals(EdocOperation.STEPBACK)){
						String backInfo = retEdocObject.getOpinion();
						detailLog.setBackOpinion(backInfo);
						detailLog.setDescription("进行了回退操作，交换状态为已回退");
						detailLog.setUserName(ocipMember.getName());
						detailLog.setStatus(ExchangeDetailStatus.hasBack.getKey());
						detail.setRecNo(null);
						detail.setRecUserName(null);
						detail.setRecUserId(null);
						detail.setRecTime(null);
						detail.setStatus(ExchangeDetailStatus.hasBack.getKey());
						Long createId = Long.valueOf(organizationManager.getPlatformId(creater));
						V3xOrgMember createMember = orgManager.getMemberById(createId);
						GovdocExchangeMain main = govdocExchangeManager.getGovdocExchangeMainById(detail.getMainId());
						CtpAffair affair = affairManager.get(main.getAffairId());
						MessageReceiver messageReceiver = new MessageReceiver(detail.getId(), main.getStartUserId());

						// 0-意见隐藏，-1内容为空，1有内容
						int opinionType = 1;
						if (Strings.isBlank(backInfo) || "null".equalsIgnoreCase(backInfo)) {
							opinionType = -1;
						}
						// -1:无附件或意见被隐藏; 1: 无内容（内容前面加“意见：”）; 2:有内容，有附件
						int opinionAtt = (Strings.isBlank(backInfo) ? 1 : 2);
						// 有内容，有附件：减少4个字节
						int deviation = opinionAtt == 2 ? -4 : 0;
						String opinionContent = UserMessageUtil.getComment4Message(backInfo, deviation);
						MessageContent mc = new MessageContent("govdoc.message.stepBack", affair.getSubject(), createMember.getName(), 0, null, opinionContent, opinionType, opinionAtt);
						mc.setImportantLevel(affair.getImportantLevel());
						mc.setBody(backInfo, com.seeyon.ctp.common.constants.Constants.EDITOR_TYPE_HTML, new Date());
						userMessageManager.sendSystemMessage(mc, ApplicationCategoryEnum.edoc, createMember.getId(), messageReceiver);
					}
					if(retEdocObject.getOperation().equals(EdocOperation.HASTEN)){//催办
						String pressInfo = retEdocObject.getOpinion();//催办信息
						V3xOrgMember v3xmember = orgManager.getMemberById(Long.parseLong(ocipMember.getId()));
						User user = new User();
						user.setId(Long.parseLong(ocipMember.getId()));
						user.setLoginAccount(v3xmember.getOrgAccountId());
						user.setAccountId(v3xmember.getOrgAccountId());
						user.setDepartmentId(v3xmember.getOrgDepartmentId());
						user.setName(v3xmember.getName());
						CommonUtil.setCurrentUser(user);
						//查出所有的待办进行催办信息的发送
						List<CtpAffair> affairList = affairManager.getAffairs(detail.getSummaryId(),StateEnum.col_pending);
						EdocSummary summary = govdocSummaryManager.getSummaryById(detail.getSummaryId());
				        for(CtpAffair affair : affairList){
				        	 if(affair.getState() == StateEnum.col_pending.getKey()){
				        		 List<String> personIds = new ArrayList<String>();
				        		 personIds.add(String.valueOf(affair.getMemberId()));
				        		 wapi.hasten(String.valueOf(summary.getProcessId()), String.valueOf(affair.getActivityId()), "edoc", personIds, null, pressInfo, false);
				        	 }
				        }
					}
				}
				if(!retEdocObject.getOperation().equals(EdocOperation.REVOKED)||!retEdocObject.getOperation().equals(EdocOperation.HASTEN)){
					govdocExchangeManager.updateDetail(detail);
					govdocExchangeManager.saveDetailLog(detailLog);
				}
			} catch (BusinessException e) {
				String msg = "ocip接受公文回执信息执行失败";
				LOGGER.error(msg,e);
				for (BussinessResult bussinessResult : bussinessResults) {
					bussinessResult.setCode("2");
					bussinessResult.setMessage(msg);
				}
			}
		}
		return bussinessResults;
	}


	@Override
	public BIZExchangeData exchangeSend(Map<String, Object> params)
			throws BussinessException {
		Address createUser = new Address();
		String localSystemCode = Global.getConfig("sysCode");
		createUser.setResource(localSystemCode);// 本系统注册id值
		createUser.setType(IConstant.AddressType.system.name());
		createUser.setId(localSystemCode);
		createUser.setName(localSystemCode);// 本系统名称
		BIZExchangeData bizData = new BIZExchangeData();
		Long detailIdStr = (Long)params.get("detailId");
		EdocOperation  edocOperation = (EdocOperation)params.get("edocOperation");
		String ocipSendStatus = params.get("ocipSendStatus")!=null?(String)params.get("ocipSendStatus"):null;
		bizData.setIdentifier(UUID.randomUUID().toString().replaceAll("-", ""));
		
		BIZMessage bizMessage = new BIZMessage();
		bizMessage.setContentType(BIZContentType.RET);
		RETEdocObject retEdocObject = new RETEdocObject();
		retEdocObject.setOperation(edocOperation);
		retEdocObject.setOriginMessageId(UUID.randomUUID().toString().replaceAll("-", ""));
		bizMessage.setContent(retEdocObject);
		User user = AppContext.getCurrentUser();
		List<Organization> receivers = new ArrayList<Organization>();
		try {
			GovdocExchangeDetail govdocExchangeDetail = govdocExchangeManager.getExchangeDetailById(detailIdStr);
					// 匹配哪个签收数据明细
			if (null!=govdocExchangeDetail) {
				Address ocipsys = (Address) BeanUtils.cloneBean(createUser);
				createUser.setName(govdocExchangeDetail.getRecOrgName());
				createUser.setId(govdocExchangeDetail.getRecOrgId().toString());
				if(govdocExchangeDetail.getRecOrgType()==GovdocEnum.GovdocExchangeOrgType.account.getKey()){
					createUser.setType(IConstant.AddressType.account.name());
				}else{
					createUser.setType(IConstant.AddressType.department.name());
				}
				GovdocExchangeMain main = govdocExchangeManager.getGovdocExchangeMainById(govdocExchangeDetail.getMainId());
				bizData.setSubject(main.getSubject());
				bizData.setGroupId(main.getSummaryId().toString());
				retEdocObject.setDetailId(detailIdStr.toString());
				retEdocObject.setOcipSyscode(ocipsys.toString());
				if(ocipSendStatus!=null){
					retEdocObject.setExchangeStatus(ocipSendStatus);
				}
				Map<String,Object> map = GovdocUtil.getExtProperty(govdocExchangeDetail);
				retEdocObject.setOriginMessageId((String)map.get("identifier"));
				V3xOrgEntity entity = orgManager.getEntity(getTypeAndIds(GovdocEnum.GovdocExchangeOrgType.account.getKey(), govdocExchangeDetail.getSendAccountId()));
				Address receiver = new Address();
				Organization reciverOrg = new Organization();
				if (entity != null && OrgUtil.isPlatformEntity(entity)) {
					receiver = OrgUtil.getAddress(entity);
				}else if(EdocOperation.REVOKED.equals(retEdocObject.getOperation())||EdocOperation.HASTEN.equals(retEdocObject.getOperation())){
					V3xOrgEntity revokeEntity = orgManager.getEntity(getTypeAndIds(govdocExchangeDetail.getRecOrgType(), govdocExchangeDetail.getRecOrgId()));
					receiver = OrgUtil.getAddress(revokeEntity);
				}
				reciverOrg.setIdentification(receiver);
				if(entity != null){
					reciverOrg.setName(entity.getName());
				}				
				receivers.add(reciverOrg);
				if(user!=null&&edocOperation!=EdocOperation.RECEIVED){//如果不是回执Recieved类型
					createUser.setName(user.getName());
					createUser.setType(AddressType.member.name());
					createUser.setId(user.getId().toString());
					String comment = (String)params.get("comment");
					if(Strings.isNotBlank(comment)){
						retEdocObject.setOpinion(comment);
					}
					if(edocOperation.equals(EdocOperation.ACCEPTED)){ //如果为签收，sender为member
						retEdocObject.setRecNo(govdocExchangeDetail.getRecNo());
					}
					
				}
			}
		}catch (Exception e) {
		}
		//发送源地址信息
		Address source = new Address();
		source.setId(localSystemCode);
		source.setName(localSystemCode);
		//发送方
		Organization sender = new Organization();
		sender.setIdentification(createUser);
		sender.setName(createUser.getName());
		bizData.setSender(sender);
		bizData.setSource(source);
		bizData.setBussnissMessage(bizMessage);
		bizData.setRecivers(receivers);
		return bizData;
	}
	private static String getTypeAndIds(int recOrgType, Long id) {
		String typeAndIds = "";
		if (GovdocEnum.GovdocExchangeOrgType.department.getKey() == recOrgType) {
			typeAndIds = "Department|" + id;
		} else if (GovdocEnum.GovdocExchangeOrgType.account.getKey() == recOrgType) {
			typeAndIds = "Account|" + id;
		}
		return typeAndIds;
	}
	private String getAffairId(GovdocExchangeDetail detail) throws BusinessException {
		Long id = detail.getSummaryId();
		List<String> processId = new ArrayList<String>();
		String affairId = null;
		List<Long> startAffairId = new ArrayList<Long>();
		List<Long> validSummaryId = new ArrayList<Long>();
		List<Long> error = new ArrayList<Long>();
		List<CtpAffair> affairs = affairManager.getAffairs(ApplicationCategoryEnum.edoc, id);
		CtpAffair pendingaffair = null;
		CtpAffair startaffair = null;
		if (affairs != null && affairs.size() > 0) {

			for (CtpAffair a : affairs) {
				if (a.getState() == StateEnum.col_pending.getKey()) {
					pendingaffair = a;
				} else if (a.getState() == StateEnum.col_sent.getKey() || a.getState() == StateEnum.col_waitSend.getKey()) {
					CommonUtil.userLogin(orgManager, orgManager.getMemberById(a.getSenderId()).getOrgAccountId());// 模拟登陆
					startaffair = a;
				}
			}
		}
		EdocSummary summary = govdocSummaryManager.getSummaryById(Long.valueOf(id));
		if (pendingaffair != null && summary != null) {
			processId.add(summary.getProcessId());
			affairId = pendingaffair.getId().toString();
			if(startaffair != null){
				startAffairId.add(startaffair.getId());
			}			
			validSummaryId.add(id);
		} else if (startaffair != null && affairs.size() == 1) {
			if(summary != null){
				processId.add(summary.getProcessId());
			}			
			affairId = startaffair.getId().toString();
			startAffairId.add(startaffair.getId());
			validSummaryId.add(id);
		} else {
			error.add(id);
		}
		return affairId;
	}

	public WorkflowApiManager getWapi() {
		return wapi;
	}

	public void setWapi(WorkflowApiManager wapi) {
		this.wapi = wapi;
	}

}
