package com.seeyon.apps.ocip.exchange.edoc;

import java.util.ArrayList;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.StringTokenizer;
import java.util.UUID;

import com.seeyon.apps.govdoc.manager.GovdocOCIPExchangeManager;
import com.seeyon.apps.ocip.OCIPConstants;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.usermessage.MessageReceiver;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.BeanUtils;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ocip.common.IConstant;
import com.seeyon.ocip.common.entry.Address;
import com.seeyon.ocip.common.org.OcipOrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.common.utils.Global;
import com.seeyon.ocip.exchange.api.IBussinessHandler;
import com.seeyon.ocip.exchange.exceptions.BussinessException;
import com.seeyon.ocip.exchange.model.BIZContentType;
import com.seeyon.ocip.exchange.model.BIZExchangeData;
import com.seeyon.ocip.exchange.model.BIZMessage;
import com.seeyon.ocip.exchange.model.BussinessResult;
import com.seeyon.ocip.exchange.model.Organization;
import com.seeyon.ocip.exchange.model.col.Affair;
import com.seeyon.ocip.exchange.model.col.ColObject;
import com.seeyon.ocip.exchange.model.col.ColOperation;
import com.seeyon.ocip.exchange.model.edoc.EdocOCIPSummary;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import org.apache.commons.logging.Log;
import org.slf4j.LoggerFactory;

public class GovdocExchangeHandler implements IBussinessHandler{
	private static final Log LOGGER = CtpLogFactory.getLog(GovdocExchangeHandler.class);
	private IOrganizationManager organizationManager;
	private GovdocOCIPExchangeManager govdocOCIPExchangeManager;
	private OrgManager orgManager;
	private boolean hasMutiPlugin = Boolean.valueOf(AppContext.getSystemProperty(OCIPConstants.HAS_MUTI_PLUGIN));//是否开启多G6功能

	public OrgManager getOrgManager() {
		return orgManager;
	}

	public void setOrgManager(OrgManager orgManager) {
		this.orgManager = orgManager;
	}

	public IOrganizationManager getOrganizationManager() {
		return organizationManager;
	}

	public void setOrganizationManager(IOrganizationManager organizationManager) {
		this.organizationManager = organizationManager;
	}

	public GovdocOCIPExchangeManager getGovdocOCIPExchangeManager() {
		return govdocOCIPExchangeManager;
	}

	public void setGovdocOCIPExchangeManager(GovdocOCIPExchangeManager govdocOCIPExchangeManager) {
		this.govdocOCIPExchangeManager = govdocOCIPExchangeManager;
	}

	@Override
	public List<BussinessResult> exchangeReceive(BIZExchangeData in) throws BussinessException {
		List<BussinessResult> reList = new ArrayList<BussinessResult>();
		//检测多G6插件
		if(!hasMutiPlugin){
			return reList;
		}
		BIZMessage bussnissMessage = in.getBussnissMessage();
		Address sourceAddress = in.getSource();
		BIZContentType contentType = bussnissMessage.getContentType();
		if (BIZContentType.EDOC.equals(contentType)) {
			// 接收内容
			ColObject content = (ColObject) bussnissMessage.getContent();
			// 操作类型
			ColOperation operation = content.getOperation();
			
			EdocOCIPSummary edocSummary = content.getEdocOCIPSummary();
				
			
			// 待办信息进行过滤
			List<Affair> affairs = content.getAffairs();

			Address sendOrg = content.getSender();
			// int opinionType = Integer.parseInt(content.getOpinionType());
			int opinionType = 1;
			String opinionContent = content.getOpinionContent();
			for (Affair affair : affairs) {
				BussinessResult result = new BussinessResult();
				result.setIdentifier(in.getIdentifier());
				Organization organization = getOrganization(in, affair);
				result.setOrganization(organization);
				// 判断是否是本系统的用户
				String userId = String.valueOf(affair.getMemberId());
				String localObjectId = organizationManager.getLocalObjectId(IConstant.AddressType.member.name(), userId);
				if (localObjectId == null) {
					// TODO 记录日志
					LOGGER.info("接收消息时转换本地人员信息失败");
					result.setCode("1");
					result.setMessage("接收消息时转换本地人员信息失败");
					reList.add(result);
					continue;
				}
				// 判断人员是否已经无效，则不生成相关的待办或处理业务
				try {
					V3xOrgMember localMemberObj = orgManager.getMemberById(Long.parseLong(localObjectId));
					if (!localMemberObj.getEnabled()) {
						LOGGER.info("接收消息时本地人员无效");
						result.setCode("1");
						result.setMessage("接收消息时本地人员无效：" + localMemberObj.getName());
						reList.add(result);
						continue;
					}
				} catch (Exception e) {
					result.setCode("2");
					result.setMessage(e.getMessage());
				}
				// 转换上一处理人
				try {
					affair.setPreApprover(Long.valueOf(content.getSender().getId()).longValue());
				} catch (Exception e) {
					LOGGER.error("发送协同交换信息时，转换上一处理人出错 ", e);
				}
				// 发起者和affair当前处理人是否同一人
				boolean isSelf = Long.valueOf(content.getSender().getId()).longValue() == affair.getMemberId();
				if (operation == ColOperation.FORWARD) {
					affair.setSenderId(Long.parseLong(localObjectId));
				}
				affair.setMemberId(Long.parseLong(localObjectId));
				try {
					// 推送类型判断
					if (ColOperation.TODO == operation) {// 推送待办
						govdocOCIPExchangeManager.sendEdocAndMeg(affair, edocSummary, new MessageReceiver(affair.getId(), affair.getMemberId().longValue(), "message.link.govdoc.pending", new Object[] { affair.getId().toString() }), sendOrg.getName(),
								opinionType, opinionContent, sourceAddress.toString());
					} else if (ColOperation.DONE == operation) {// 修改Affair和EdocSummary
						govdocOCIPExchangeManager.sendDone(affair, edocSummary, sourceAddress.toString());
					} else if (ColOperation.HASTEN == operation) {// 推送催办
					} else if (ColOperation.STOP == operation) {// 推送终止
						govdocOCIPExchangeManager.sendStop(affair, edocSummary, sourceAddress.toString(), sendOrg.getName(), opinionContent, isSelf);
					} else if (ColOperation.STEPBACK == operation) {// 推送回退
						govdocOCIPExchangeManager.sendStepBack(affair, edocSummary, sendOrg.getName(), operation, opinionContent, isSelf, sourceAddress.toString());
					} else if (ColOperation.TAKEBACK == operation) {// 推送取回
						govdocOCIPExchangeManager.sendStepBack(affair, edocSummary, sendOrg.getName(), operation, opinionContent, isSelf, sourceAddress.toString());
					} else if (ColOperation.COMPETE == operation) {// 推送竞争执行消息
						govdocOCIPExchangeManager.competitionDone(affair, edocSummary);
					} else if (ColOperation.REVOKED == operation) {// 推送撤销/回退
						govdocOCIPExchangeManager.sendStepBack(affair, edocSummary, sendOrg.getName(), operation, opinionContent, isSelf, sourceAddress.toString());
					} else if (ColOperation.COMMENTS == operation) {// 新增附言
					} else if (ColOperation.OVERTIME == operation) {// 超期提醒
					} else if (ColOperation.MODIFYBODY == operation) {// 修改正文提醒
					} else if (ColOperation.COMMENTREPLY == operation) {// 意见回复推送消息
						govdocOCIPExchangeManager.sendCommentsReply(affair, edocSummary,sendOrg.getName());
					} else if (ColOperation.FAVORITE == operation) {// 收藏
					} else if (ColOperation.FORWARD == operation) {// 转发
					} else if (ColOperation.ZCDB == operation) {// 暂存待办
						// collaborationExchangeManager.sendZcdb(affair,
						// colSummary);
					} else {
						result.setCode("-1");
						result.setMessage("操作码[" + operation.name() + "]不存在");
					}
				} catch (Exception e) {
					result.setCode("2");
					result.setMessage(e.getMessage());
					reList.add(result);
					LoggerFactory.getLogger(getClass()).error("处理平台交换数据失败", e);
					continue;
				}
				result.setCode("0");
				result.setMessage("公文接收成功");
				reList.add(result);
			}

		} else {
			throw new BussinessException();
		}
		return reList;
	}

	@Override
	public BIZExchangeData exchangeSend(Map<String, Object> param) throws BussinessException {
		String localSystemCode = Global.getSysCode();
		EdocSummary summary = (EdocSummary) param.get("edocSummary");
		ColOperation operation = (ColOperation) param.get("ColOperation");
		try {
			summary = (EdocSummary) org.apache.commons.beanutils.BeanUtils.cloneBean(summary);
		} catch (Exception e) {
			LOGGER.error("summary拷贝失败", e);
		}
		AffairData data = (AffairData) param.get("affairData");
		User currentUser = new User();
		BIZExchangeData bizData = new BIZExchangeData();
		String id = UUID.randomUUID().toString().replaceAll("-", "");
		bizData.setIdentifier(id);
		LOGGER.error(operation.name() + id);
		// 发送方当前操作者 //TODO 可能当前是代理人员
		Organization sender = new Organization();
		V3xOrgMember currentMember = null;
		try {
			currentUser = initCurrentUser(currentUser, summary, operation);
			currentMember = orgManager.getMemberById(currentUser.getId());
			Address address = OrgUtil.getAddress(currentMember);
			sender.setIdentification(address);
			sender.setName(currentMember.getName());
		} catch (BusinessException e1) {
			throw new BussinessException("封装交换数据包失败，原因：获取当前操作者失败！", e1);
		} catch (BussinessException e) {
			throw new BussinessException("封装交换数据包失败，原因：获取当前操作者失败！", e);
		}
		bizData.setSender(sender);

		// 接收者
		List<Affair> ocipAffairs = new ArrayList<Affair>();
		List<Organization> recivers = new ArrayList<Organization>();
		initRecivers(data, ocipAffairs, recivers);
		// 消息对象封装
		ColObject colObject = new ColObject();
		colObject.setAffairs(ocipAffairs);
		EdocOCIPSummary edocSummary = new EdocOCIPSummary();
		try {
			// 将本地发起人员在交换的时候替换成平台人员ID
			if (summary != null && summary.getId() != -1L) {
				OcipOrgMember sMember = OrgUtil.getPlatformMember(summary.getStartMemberId());
				if (sMember != null) {
					summary.setStartMemberId(Long.parseLong(sMember.getId()));
					summary.setStartUserId(Long.parseLong(sMember.getId()));
				}
				initCurrentNodeInfo(localSystemCode, summary);
				bizData.setGroupId(String.valueOf(summary.getId()));
			}
		} catch (Exception e) {
			LOGGER.error("将本地发起人员在交换的时候替换成平台人员ID失败", e);
		}
		BeanUtils.convert(edocSummary, summary);
		edocSummary = initColSummaryByOperation(edocSummary, operation, data);
		colObject.setEdocOCIPSummary(edocSummary);
		colObject.setOperation(operation);
		colObject.setDatetime(new Date());
		// 发起者信息
		// 如果不是平台对象，需要转换成平台ID
		try
		{
			if (!OrgUtil.isPlatformEntity(currentMember)) {
				OcipOrgMember platformMember = OrgUtil.getPlatformMember(currentMember.getId());
				currentMember.setId(Long.valueOf(platformMember.getId()));
			}
		}
		catch (Exception e)
		{
			LOGGER.error("当前人员转换失败id" + currentMember.getId(),e);
			throw new BussinessException("封装交换数据包失败，原因：转换当前人员为平台人员失败！", e);
		}
		colObject.setSender(OrgUtil.getAddress(currentMember));
		String opinionType = "0";// ?暂时没发现用处，50 发送短信
		if (data.getIsSendMessage() != null && data.getIsSendMessage()) {
			opinionType = "50";
		}
		colObject.setOpinionType(opinionType);
		Object opinionContent = param.get("opinionContent");
		colObject.setOpinionContent(opinionContent == null ? "" : opinionContent.toString());

		bizData.setRecivers(recivers);
		try {
			if (edocSummary.getSubject() == null) {
				bizData.setSubject(data.getSubject());
			} else {
				String subject = edocSummary.getSubject();
				bizData.setSubject(subject);
			}
		} catch (Exception e) {
		}
		BIZMessage bizMessage = new BIZMessage();
		bizMessage.setContentType(BIZContentType.EDOC);
		bizMessage.setContent(colObject);
		// 发送源地址信息
		Address source = new Address();
		source.setId(localSystemCode);
		source.setName(localSystemCode);
		bizData.setSource(source);
		// 传送的附件
		bizMessage.setAttachments(null);
		bizData.setBussnissMessage(bizMessage);
		return bizData;
	}
	private EdocOCIPSummary initColSummaryByOperation(EdocOCIPSummary edocOCIPSummary, ColOperation operation, AffairData data) {
		if (operation == ColOperation.COMMENTREPLY) {// 意见回复传送colSummary消息封装
			edocOCIPSummary.setCurrentNodesInfo(data.getForwardMember());// 当前处理人
			edocOCIPSummary.setStartMemberId(data.getMemberId());// 当前处理人ID
			edocOCIPSummary.setCaseId(data.getCaseId());// 存储意见回复ID
			edocOCIPSummary.setEdocType(data.getModuleType());// 意见回复commentType
			edocOCIPSummary.setSubject(data.getSubject());// 回复意见的标题
			edocOCIPSummary.setComment(data.getBodyContent());// 意见回复内容
			edocOCIPSummary.setBodyType(data.getBodyType());// 意见回复接收人姓名
		}

		return edocOCIPSummary;
	}
	@Override
	public BIZContentType type() {
		return BIZContentType.EDOC;
	}
	
	/**
	 * 查询当前发送人
	 * 
	 * @param
	 * @param operation
	 * @return
	 * @throws Exception
	 */
	private User initCurrentUser(User currentUser, EdocSummary summary, ColOperation operation) throws BussinessException {
		if (operation != ColOperation.OVERTIME) {
			currentUser = AppContext.getCurrentUser();
		} else {
			V3xOrgMember member;
			try {
				member = orgManager.getMemberById(summary.getStartMemberId());
				LOGGER.warn("创建人员" + member.getId());
				currentUser.setId(summary.getStartMemberId());
				currentUser.setLoginAccount(member.getOrgAccountId());
				currentUser.setAccountId(member.getOrgAccountId());
				currentUser.setDepartmentId(member.getOrgDepartmentId());
				currentUser.setName(member.getName());
			} catch (BusinessException e2) {
				throw new BussinessException("通过传输交换数据类型判断用户信息,原因：获取当前操作者失败！", e2);
			}

		}
		if(currentUser == null){
			currentUser = new User();
			V3xOrgMember member;
			try {
				member = orgManager.getMemberById(summary.getStartMemberId());
				LOGGER.warn("创建人员" + member.getId());
				currentUser.setId(summary.getStartMemberId());
				currentUser.setLoginAccount(member.getOrgAccountId());
				currentUser.setAccountId(member.getOrgAccountId());
				currentUser.setDepartmentId(member.getOrgDepartmentId());
				currentUser.setName(member.getName());
			} catch (BusinessException e2) {
				throw new BussinessException("通过传输交换数据类型判断用户信息,原因：获取当前操作者失败！", e2);
			}
		}
		return currentUser;
	}
	
	/**
	 * 生成消息接收者
	 * 
	 * @param data
	 * @param ocipAffairs
	 * @param recivers
	 */
	private void initRecivers(AffairData data, List<Affair> ocipAffairs, List<Organization> recivers) {
		List<CtpAffair> affairs = data.getAffairList();
		if (affairs != null && affairs.size() > 0) {
			for (CtpAffair ctpAffair : affairs) {
				OcipOrgMember reciveOcipMember = OrgUtil.getPlatformMember(ctpAffair.getMemberId());
				if (reciveOcipMember == null) {
					LOGGER.warn("处理交换数据失败，将跳过该待办数据，原因：未找到待办接收人对应的平台对象，aff = " + ctpAffair.toJSON());
					continue;
				}
				// 转换转发人
				String forwardMemberId = ctpAffair.getForwardMember();
				if (Strings.isNotEmpty(forwardMemberId)) {
					OcipOrgMember forwardMember = OrgUtil.getPlatformMember(forwardMemberId);
					if (null != forwardMember) {
						ctpAffair.setForwardMember(forwardMember.getId());
					}
				}
				// 转换加签，会签人
				if (ctpAffair.getFormId() != null) {
					OcipOrgMember fromMember = OrgUtil.getPlatformMember(ctpAffair.getFormId());
					if (null != fromMember) {
						ctpAffair.setFromId(Long.parseLong(fromMember.getId()));
					}
				}
				Organization reciverOrg = new Organization();
				Address recAdd = new Address();
				recAdd.setId(reciveOcipMember.getId());
				recAdd.setResource(IConstant.OCIP_RESOURCE);
				recAdd.setName(reciveOcipMember.getName());
				recAdd.setType(IConstant.AddressType.member.name());
				reciverOrg.setIdentification(recAdd);
				reciverOrg.setName(reciveOcipMember.getName());
				recivers.add(reciverOrg);
				ctpAffair.setMemberId(Long.parseLong(reciveOcipMember.getId()));
				OcipOrgMember sendMember = OrgUtil.getPlatformMember(ctpAffair.getSenderId());
				if (sendMember == null) {
					ctpAffair.setSenderId(ctpAffair.getSenderId());
				} else {
					ctpAffair.setSenderId(Long.parseLong(sendMember.getId()));
				}
				ctpAffair.setUpdateDate(new Date());
				ocipAffairs.add(initOcipAffair(ctpAffair));
			}
		}
	}
	/**
	 * 根据G6 affair封装 OICP affair对象
	 * 
	 * @param ctpAffair
	 * @return
	 */
	private Affair initOcipAffair(CtpAffair ctpAffair) {
		Affair ocipAffair = new Affair();
		BeanUtils.convert(ocipAffair, ctpAffair);
		return ocipAffair;
	}
	
	private void initCurrentNodeInfo(String localSystemCode, EdocSummary summary) {
		String currentNodesInfo = summary.getCurrentNodesInfo();
		if (Strings.isNotEmpty(currentNodesInfo)) {
			List<String> currentNodesNew = new ArrayList<String>();
			StringTokenizer tokenizer = new StringTokenizer(currentNodesInfo, ";");
			while (tokenizer.hasMoreTokens()) {
				String next = tokenizer.nextToken();
				OcipOrgMember currentMember = null;
				try {
					currentMember = OrgUtil.getPlatformMember(Long.parseLong(next));
				} catch (Exception e) {
				}
				if (null != currentMember) {
					currentNodesNew.add(currentMember.getId());
				} else {
					currentNodesNew.add(next);
				}
			}
			summary.setCurrentNodesInfo(Strings.join(currentNodesNew, ";"));
		}
	}
	private Organization getOrganization(BIZExchangeData in, Affair affair) {
		Organization organization = null;
		for (Organization o : in.getRecivers()) {
			if (o.getIdentification().getId().equals(affair.getMemberId().toString())) {
				organization = o;
				break;
			}
		}
		return organization;
	}
}
