package com.seeyon.apps.ocip.exchange.edoc;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import com.seeyon.apps.govdoc.manager.GovdocOCIPExchangeManager;
import com.seeyon.apps.govdoc.manager.GovdocSummaryManager;
import com.seeyon.v3x.edoc.domain.EdocSummary;
import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.logging.Log;
import org.aspectj.lang.JoinPoint;

import com.seeyon.apps.ocip.OCIPConstants;
import com.seeyon.apps.ocip.util.OrgUtil;
import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.manager.AffairManager;
import com.seeyon.ctp.common.affair.util.AffairUtil;
import com.seeyon.ctp.common.authenticate.domain.User;
import com.seeyon.ctp.common.constants.ApplicationCategoryEnum;
import com.seeyon.ctp.common.content.affair.AffairData;
import com.seeyon.ctp.common.content.affair.constants.StateEnum;
import com.seeyon.ctp.common.content.comment.Comment;
import com.seeyon.ctp.common.content.comment.CommentManager;
import com.seeyon.ctp.common.exceptions.BusinessException;
import com.seeyon.ctp.common.log.CtpLogFactory;
import com.seeyon.ctp.common.po.affair.CtpAffair;
import com.seeyon.ctp.common.po.comment.CtpCommentAll;
import com.seeyon.ctp.common.track.enums.TrackEnum;
import com.seeyon.ctp.common.track.manager.CtpTrackMemberManager;
import com.seeyon.ctp.common.track.po.CtpTrackMember;
import com.seeyon.ctp.common.usermessage.UserMessageUtil;
import com.seeyon.ctp.organization.bo.V3xOrgMember;
import com.seeyon.ctp.organization.manager.OrgManager;
import com.seeyon.ctp.util.Strings;
import com.seeyon.ctp.util.UniqueList;
import com.seeyon.ocip.common.org.OrgMember;
import com.seeyon.ocip.common.organization.IOrganizationManager;
import com.seeyon.ocip.common.utils.Global;
import com.seeyon.ocip.exchange.model.col.ColOperation;
import com.seeyon.ocip.online.OnlineChecker;

public class GovdocMessageExchangeManagerImpl implements GovdocMessageExchangeManager {
    private static Log LOG = CtpLogFactory.getLog(GovdocMessageExchangeManagerImpl.class);
    private boolean hasMutiPlugin = Boolean.valueOf(AppContext.getSystemProperty(OCIPConstants.HAS_MUTI_PLUGIN));//是否开启多G6功能
    private AffairManager affairManager;
    private OrgManager orgManager;

    private CommentManager ctpCommentManager;
    private GovdocOCIPExchangeManager govdocOCIPExchangeManager;
    private CtpTrackMemberManager trackManager;
    private IOrganizationManager organizationManager;
    private GovdocSummaryManager govdocSummaryManager;
    public void setAffairManager(AffairManager affairManager) {
        this.affairManager = affairManager;
    }

    public void setOrgManager(OrgManager orgManager) {
        this.orgManager = orgManager;
    }

    public void setGovdocSummaryManager(GovdocSummaryManager govdocSummaryManager) {
        this.govdocSummaryManager = govdocSummaryManager;
    }

    public void setGovdocOCIPExchangeManager(GovdocOCIPExchangeManager govdocOCIPExchangeManager) {
        this.govdocOCIPExchangeManager = govdocOCIPExchangeManager;
    }

    public void setTrackManager(CtpTrackMemberManager trackManager) {
        this.trackManager = trackManager;
    }

    public void setOrganizationManager(IOrganizationManager organizationManager) {
        this.organizationManager = organizationManager;
    }

    public void setCtpCommentManager(CommentManager ctpCommentManager) {
        this.ctpCommentManager = ctpCommentManager;
    }

    @Override
    public void doCommentPushExchangeMessage4SenderNote(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            Comment comment = (Comment) joinPoint.getArgs()[0];
            Long summaryId = comment.getModuleId();
            try {
                List<CtpAffair> affairList = affairManager.getValidAffairs(ApplicationCategoryEnum.collaboration, summaryId);
                if (affairList == null || affairList.isEmpty()) {
                    return;
                }
                List<CtpAffair> list = new UniqueList<CtpAffair>(); //用于存储提醒外系统人员意见回复事务
                for (CtpAffair affair : affairList) {
                    Long memberId = affair.getMemberId();
                    Long senderId = affair.getSenderId();
                    if (memberId.intValue() == senderId.intValue()) {
                        continue;
                    }
                    //判断新增附言发送人员是否为外单位
                    try {
                        V3xOrgMember dMember = orgManager.getMemberById(memberId);
                        if (OrgUtil.isPlatformEntity(dMember)) {
                            CtpAffair addAffair = (CtpAffair) BeanUtils.cloneBean(affair);
                            addAffair.setSubject(comment.getContent());
                            addAffair.setTempleteId(comment.getId());
                            addAffair.setAddition(comment.getTitle());
                            list.add(addAffair);
                        }
                    } catch (Exception e) {

                    }
                }
                //判断是否有外单位人员接收消息
                try {
                    if (!list.isEmpty()) {
                        AffairData affairData = new AffairData();
                        affairData.setAffairList(list);
                        affairData.setModuleId(comment.getModuleId());
                        affairData.setIsSendMessage(false);
                        affairData.setSubject(comment.getContent());
                        EdocSummary summary = govdocSummaryManager.getSummaryById(summaryId);
                        govdocOCIPExchangeManager.handCollaborationAffair(affairData, ColOperation.COMMENTS, summary);
                    }
                } catch (Exception e) {
                    LOG.error(e);
                }
            } catch (BusinessException e) {
                LOG.error(e);
            }
        }
    }

    @Override
    public void sendPushExhangeMessage4Reply(JoinPoint joinPoint) {
        if (AppContext.hasPlugin("ocip")) {
            User user = AppContext.getCurrentUser();
            Comment comment = (Comment) joinPoint.getArgs()[0];
            Long summaryId = comment.getModuleId();
            try {
                CtpCommentAll comAll = ctpCommentManager.getById(comment.getPid());
                String memberName = orgManager.getMemberById(comAll.getCreateId()).getName();
                String commentContent = UserMessageUtil.getComment4Message(Strings.toText(comment.getContent()));
                int commentType = Strings.isTrue(comment.isHidden()) ? 0 : Strings.isBlank(commentContent) ? -1 : 1;
                List<CtpAffair> trackingAffairList = affairManager.getValidTrackAffairs(summaryId);
                List<CtpTrackMember> trackMembers = trackManager.getTrackMembers(summaryId, null);
                trackingAffairList = getTrackAffairExcludePart(trackingAffairList, trackMembers, user.getId());
                List<Long[]> pushMemberIds = comment.getPushMessageToMembersList();
                if (trackingAffairList.isEmpty() && pushMemberIds.isEmpty()) {
                    return;
                }
                String subject = comment.getTitle();
                //用于存储提醒外系统人员意见回复事务
                List<CtpAffair> list = new UniqueList<CtpAffair>();
                List<String> objectList = new ArrayList<String>();
                Map<String, List<CtpAffair>> localAffair = new HashMap<String, List<CtpAffair>>();
                for (CtpAffair taffair : trackingAffairList) {
                    if (taffair.isDelete()) {
                        continue;
                    }
                    Long recieverMemberId = taffair.getMemberId();
                    //判断人员是不是提醒外系统人员,用于修改正文协同交换提醒
                    try {
                        V3xOrgMember dMember = orgManager.getMemberById(recieverMemberId);
                        if (OrgUtil.isPlatformEntity(dMember)) {
                            CtpAffair addAffair = (CtpAffair) BeanUtils.cloneBean(taffair);
                            list.add(addAffair);
                        } else {
                            CtpAffair addAffair = (CtpAffair) BeanUtils.cloneBean(taffair);
                            if (localAffair.containsKey(dMember.getId().toString())) {
                                localAffair.get(dMember.getId().toString()).add(addAffair);
                            } else {
                                List<CtpAffair> listAff = new ArrayList<CtpAffair>();
                                listAff.add(addAffair);
                                localAffair.put(dMember.getId().toString(), listAff);
                            }
                            objectList.add(dMember.getId().toString());
                        }
                    } catch (Exception e) {

                    }
                }
                for (Long[] push : pushMemberIds) {
                    //判断人员是不是提醒外系统人员,用于修改正文协同交换提醒
                    try {
                        V3xOrgMember dMember = orgManager.getMemberById(push[1]);
                        if (OrgUtil.isPlatformEntity(dMember)) {
                            CtpAffair commentAffair = affairManager.get(push[0]);
                            commentAffair.setMemberId(push[1]);
                            list.add(commentAffair);
                        } else {
                            CtpAffair commentAffair = affairManager.get(push[0]);
                            commentAffair.setMemberId(push[1]);
                            if (localAffair.containsKey(dMember.getId().toString())) {
                                localAffair.get(dMember.getId().toString()).add(commentAffair);
                            } else {
                                List<CtpAffair> listAff = new ArrayList<CtpAffair>();
                                listAff.add(commentAffair);
                                localAffair.put(dMember.getId().toString(), listAff);
                            }
                            objectList.add(dMember.getId().toString());
                        }
                        //多G6
                        if (!objectList.isEmpty() && organizationManager != null && hasMutiPlugin && OnlineChecker.isOnline()) {
                            String resource = Global.getConfig("sysCode");
                            Map<String, List<OrgMember>> mappingMember = organizationManager.getMemberByObjectIds(objectList,
                                    resource);
                            for (String objectId : objectList) {
                                List<OrgMember> memberList = mappingMember.get(objectId);
                                if (memberList != null && memberList.size() > 1) {
                                    for (CtpAffair cloneBean : localAffair.get(objectId)) {
                                        Map<String, Object> affairExtMap = AffairUtil.getExtProperty(cloneBean);
                                        affairExtMap.put("memberId_of_ocip_client", cloneBean.getMemberId());
                                        cloneBean.setMemberId(Long.valueOf(memberList.get(0).getOrgPlatformUserId()));
                                        list.add(cloneBean);
                                    }
                                }
                            }
                        }
                    } catch (Exception e) {

                    }
                }
                try {
                    //跨系统对意见回复进行提醒
                    if (!list.isEmpty()) {
                        AffairData affairData = new AffairData();
                        affairData.setAffairList(list);
                        affairData.setIsSendMessage(false);
                        affairData.setSubject(subject);
                        affairData.setMemberId(user.getId());//借用存储当前意见回复人员
                        affairData.setForwardMember(user.getName());//借用存储当前意见回复人员姓名
                        affairData.setCaseId(comment.getId());//借用存储意见回复ID
                        affairData.setModuleType(commentType);//借用存储意见回复commentType
                        affairData.setBodyContent(commentContent);//借用存储意见回复内容
                        affairData.setBodyType(memberName);//借用存储意见回复接收人姓名
                        EdocSummary summary = govdocSummaryManager.getSummaryById(summaryId);
                        govdocOCIPExchangeManager.handCollaborationAffair(affairData, ColOperation.COMMENTREPLY, summary);
                    }
                } catch (Exception e) {

                }
            } catch (BusinessException e) {
                LOG.error(e);
            }

        }
    }

    @Override
    public void sendExchangeMessage4Zcdb(JoinPoint joinPoint) {
//        if (AppContext.hasPlugin("ocip")) {
//            CtpAffair zcdbAffair = (CtpAffair) joinPoint.getArgs()[0];
//            //判断是否外单位人员操作终止操作
//            List<CtpAffair> list = new UniqueList<CtpAffair>();
//            try {
//                V3xOrgMember dMember = orgManager.getMemberById(zcdbAffair.getMemberId());
//                if (OrgUtil.isPlatformEntity(dMember)) {
//                    CtpAffair cloneBean = (CtpAffair) BeanUtils.cloneBean(zcdbAffair);
//                    list.add(cloneBean);
//                }
//                if (!list.isEmpty()) {
//                    ColSummary colSummary = colManager.getColSummaryById(list.get(0).getObjectId());
//                    AffairData affairData = new AffairData();
//                    affairData.setAffairList(list);
//                    affairData.setSubject(colSummary.getSubject());
//                    affairData.setIsSendMessage(false);
//                    govdocOCIPExchangeManager.handCollaborationAffair(affairData, ColOperation.ZCDB, colSummary);
//                }
//            } catch (Exception e) {
//
//            }
//        }
    }

    @Override
    public void sendExchangeMessage4ModifyBodyOrAtt(JoinPoint joinPoint) {
//        if (AppContext.hasPlugin("ocip")) {
//            ColSummary summary = (ColSummary) joinPoint.getArgs()[0];
//            CtpAffair af = (CtpAffair) joinPoint.getArgs()[3];
//            User user = AppContext.getCurrentUser();
//            //获取当前事项的affair集合
//            List<StateEnum> states = new ArrayList<StateEnum>();
//            states.add(StateEnum.col_pending);
//            states.add(StateEnum.col_done);
//            states.add(StateEnum.col_sent);
//            try {
//                List<CtpAffair> affairs = new ArrayList<CtpAffair>();
//                if (af == null) {
//                    affairs = affairManager.getAffairs(af, states);
//                } else {
//                    affairs = affairManager.getAffairs(summary.getId(), states);
//                }
//                //节点人和跟踪节点需要发送消息的人员集合
//                List<Long> receiversList = new UniqueList<Long>();
//                //用于存储提醒外系统人员超期事务
//                List<CtpAffair> list = new UniqueList<CtpAffair>();
//                for (CtpAffair affair : affairs) {
//                    if (affair.isDelete()) {
//                        continue;
//                    }
//                    //获取代理人
//                    Long agentMemberId = MemberAgentBean.getInstance().getAgentMemberId(ApplicationCategoryEnum.collaboration.key(), affair.getMemberId());
//                    if (user.getId().equals(agentMemberId)) {
//                        if (!receiversList.contains(affair.getMemberId())) {
//                            receiversList.add(affair.getMemberId());
//                            //判断人员是不是提醒外系统人员,用于修改正文协同交换提醒
//                            try {
//                                V3xOrgMember dMember = orgManager.getMemberById(affair.getMemberId());
//                                if (OrgUtil.isPlatformEntity(dMember)) {
//                                    CtpAffair addAffair = (CtpAffair) BeanUtils.cloneBean(affair);
//                                    list.add(addAffair);
//                                }
//                            } catch (Exception e) {
//
//                            }
//                        }
//                        continue;
//                    }
//                    //给已发节点发送消息
//                    Long sendId = affair.getSenderId();
//                    if (Integer.valueOf(StateEnum.col_sent.getKey()).equals(affair.getState()) && !receiversList.contains(affair.getSenderId())) {
//                        receiversList.add(sendId);
//                        try {
//                            V3xOrgMember dMember = orgManager.getMemberById(sendId);
//                            if (OrgUtil.isPlatformEntity(dMember)) {
//                                //对于提醒发起人，标识是提醒发起人
//                                CtpAffair cloneBean = (CtpAffair) BeanUtils.cloneBean(affair);
//                                list.add(cloneBean);
//                            }
//                        } catch (Exception e) {
//
//                        }
//                    } else if (!receiversList.contains(affair.getMemberId())) {
//                        receiversList.add(affair.getMemberId());
//                        //判断人员是不是提醒外系统人员,用于修改正文协同交换提醒
//                        try {
//                            V3xOrgMember dMember = orgManager.getMemberById(affair.getMemberId());
//                            if (OrgUtil.isPlatformEntity(dMember)) {
//                                CtpAffair cloneBean = (CtpAffair) BeanUtils.cloneBean(affair);
//                                list.add(cloneBean);
//                            }
//                        } catch (Exception e) {
//
//                        }
//                    }
//                }
//                try {
//                    //跨系统对超期提醒人员进行分发
//                    if (!list.isEmpty()) {
//                        AffairData affairData = new AffairData();
//                        affairData.setAffairList(list);
//                        affairData.setIsSendMessage(false);
//                        affairData.setSubject("修改正文提醒");
//                        affairData.setMemberId(user.getId());//借用存储当前修改正文人员
//                        affairData.setForwardMember(user.getName());//借用存储当前修改正文人员姓名
//                        govdocOCIPExchangeManager.handCollaborationAffair(affairData, ColOperation.MODIFYBODY, summary);
//                    }
//                } catch (Exception e) {
//
//                }
//            } catch (BusinessException e) {
//                LOG.error(e);
//            }
//        }
    }

    //查找哪些设置了部分跟踪但是没有跟踪当前用户的Affair,从trackAffairs中移除，然后返回跟踪事项列表
    private List<CtpAffair> getTrackAffairExcludePart(List<CtpAffair> trackAffairs, List<CtpTrackMember> trackMembers, Long currentMemberId) {
        for (Iterator<CtpAffair> it = trackAffairs.iterator(); it.hasNext(); ) {
            CtpAffair affair = it.next();
            boolean partTrack = Integer.valueOf(TrackEnum.part.ordinal()).equals(affair.getTrack()); //设置的是部分跟踪
            boolean isTrackCurrentMemebr = false; //设置部分跟踪的时候是否跟踪了当前的用户
            for (CtpTrackMember colTrackMember : trackMembers) {
                if (affair.getId().equals(colTrackMember.getAffairId())) {
                    partTrack = true;
                    if (currentMemberId.equals(colTrackMember.getTrackMemberId())) {
                        isTrackCurrentMemebr = true;
                    }
                }
            }
            //设置了部分跟踪但是没有跟踪当前用户的
            if (partTrack && !isTrackCurrentMemebr) {
                it.remove();
            }
        }
        return trackAffairs;
    }
}
