package com.seeyon.apps.ocip.exchange.edoc;

import java.util.Locale;
import java.util.Map;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import com.seeyon.ctp.common.AppContext;
import com.seeyon.ctp.common.affair.util.WFComponentUtil;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.web.servlet.LocaleResolver;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.View;
import org.springframework.web.servlet.ViewResolver;
import org.springframework.web.servlet.handler.HandlerInterceptorAdapter;

/**
 * Ocip 公文交换，穿透拦截器
 *
 * @author renx 2019-4-23
 * @since ocip v2.0
 */
public class OcipEdocURLInterceptor extends HandlerInterceptorAdapter {

    private static final Logger logger = LoggerFactory.getLogger(OcipEdocURLInterceptor.class);

    private IOcipSSOGovdoc ocipSSOGovdoc;

    public void setOcipSSOGovdoc(IOcipSSOGovdoc ocipSSOGovdoc) {
        this.ocipSSOGovdoc = ocipSSOGovdoc;
    }

    @Override
    public boolean preHandle(HttpServletRequest request, HttpServletResponse response, Object handler) throws Exception {
        String method = request.getParameter("method");
        if ("summary".equals(method)) {
            // 用于判断是新建页面打开控件还是处理界面打开
            try {
                ModelAndView modelAndView = ocipSSOGovdoc.getOutEdocLink(request);
                if (modelAndView == null) {
                    modelAndView = ocipSSOGovdoc.getGovdocModel(request);
                }
                if (modelAndView != null) {
                    Map<String, Object> model = modelAndView.getModel();
                    String error = (String) model.get("error");
                    if (error != null) {
                        WFComponentUtil.webAlertAndClose(response, error);
                        return false;
                    }
                    ViewResolver viewResolver = (ViewResolver) AppContext.getBean("viewResolver");
                    LocaleResolver localeResolver = (LocaleResolver) AppContext.getBean("localeResolver");
                    Locale locale = Locale.getDefault();
                    if (localeResolver != null) {
                        locale = localeResolver.resolveLocale(request);
                    }
                    View view = viewResolver.resolveViewName(modelAndView.getViewName(), locale);
                    view.render(modelAndView.getModel(), request, response);
                    return false;
                }
            } catch (Exception e) {
                logger.error("", e);
                WFComponentUtil.webAlertAndClose(response, "无法访问该公文,可能本地代理关闭导致！");
                return false;
            }
        }
        return super.preHandle(request, response, handler);
    }


}
